@@ -12,10 +12,9 @@ import py_compile
 import shutil
 import glob
 import os.path
+from _igeCore import *
 import igeCore as core
 from igeCore import apputil
-from igeCore._igeCore import *
-from igeCore import apputil
 from igeCore.apputil import launch_server
 import hashlib
 import tokenize
