@@ -1,3 +1,33 @@
+//figure
+PyDoc_STRVAR(figure_doc,
+	"3D scene object.\n"\
+	"\n"\
+	"igeCore.figure(name, cloneSkeleton, cloneMesh)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"	name : string\n"\
+	"		Unique name of figure\n"\
+	"	cloneSkeleton : bool\n"\
+	"		Determine if each instance has its own Skeleton information.\n"\
+	"	cloneMesh : bool\n"\
+	"		Determine if each instance has its own Mesh information.\n"\
+	"Note\n"\
+	"----------\n"\
+	"	Normally, when you generate multiple figures from the same file,	\n"\
+	"	Skeleton and Mesh informations are shared for data saving.	\n"\
+	"	Usually it's no probrem,	\n"\
+	"	But, if you use the set/getJoint() function 	\n"\
+	"	to directly manipulate the skeleton,	\n"\
+	"	Editing one figure will affect the other figures as well.	\n"\
+	"	Enabling 'cloneSkeleton' will generate the Skeleton information separately, 	\n"\
+	"	which will aboid this issue.	\n"\
+	"	Similarly, if you use the setVertexElements() function to 	\n"\
+	"	edit the vertex directly, 	\n"\
+	"	you will get similar problems.	\n"\
+	"	enabling 'cloneMesh' will aboid it.	\n");
+
+
 
 //position
 PyDoc_STRVAR(position_doc,
@@ -208,29 +238,7 @@ PyDoc_STRVAR(getBlendingWeight_doc,
 PyDoc_STRVAR(getJoint_doc,
 	"Get the value of joint after animation calculation in world coordinate system\n"\
 	"\n"\
-	"position, rotation, scale = figure.getJoint(jointName or index)\n"\
-	"\n"\
-	"Parameters\n"\
-	"----------\n"\
-	"    jointName : string\n"\
-	"        The name of the joint you want to get\n"\
-	"    index : int\n"\
-	"        The index of the joint you want to get\n"\
-	"Returns\n"\
-	"-------\n"\
-	"    position\n"\
-	"        position of joint\n"\
-	"    rotation\n"\
-	"        rotation of joint\n"\
-	"    scale\n"\
-	"        scale of joint");
-
-
-//setJoint
-PyDoc_STRVAR(setJoint_doc,
-	"Set the value of joint in world coordinate system\n"\
-	"\n"\
-	"figure.setJoint(jointName or index, position, rotation, scale)\n"\
+	"position, rotation, scale = figure.getJoint(jointName or jointIndex, space)\n"\
 	"\n"\
 	"Parameters\n"\
 	"----------\n"\
@@ -238,14 +246,42 @@ PyDoc_STRVAR(setJoint_doc,
 	"        The name of the joint you want to get\n"\
 	"    jointIndex : int\n"\
 	"        The index of the joint you want to get\n"\
-	"    position\n"\
+	"	space : int (optional)	\n"\
+	"		Coordinate system of the value to get\n	"\
+	"       (core.WorldSpace or core.LocalSpace)\n	"\
+	"Returns\n"\
+	"-------\n"\
+	"    position : igeVmath.Vec3\n"\
 	"        position of joint\n"\
-	"    rotation\n"\
+	"    rotation : igeVmath.Quat\n"\
 	"        rotation of joint\n"\
-	"    scale\n"\
+	"    scale : igeVmath.Vec3\n"\
 	"        scale of joint");
 
 
+//setJoint
+PyDoc_STRVAR(setJoint_doc,
+	"Set the value of joint in world coordinate system\n"\
+	"\n"\
+	"figure.setJoint(jointName or jointIndex, position, rotation, scale, space)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"    jointName : string\n"\
+	"        The name of the joint you want to get\n"\
+	"    jointIndex : int\n"\
+	"        The index of the joint you want to get\n"\
+	"    position : igeVmath.Vec3 (optional)\n"\
+	"        position of joint\n"\
+	"    rotation : igeVmath.Quat (optional)\n"\
+	"        rotation of joint\n"\
+	"    scale : igeVmath.Vec3 (optional)\n"\
+	"        scale of joint\n"\
+	"	space : int (optional)	\n"\
+	"		Coordinate system of the value to get\n	"\
+	"       (core.WorldSpace or core.LocalSpace)\n	");
+
+
 //jointNameToIndex
 PyDoc_STRVAR(jointNameToIndex_doc,
 	"Get Joint index from Joint name\n"\
@@ -264,22 +300,20 @@ PyDoc_STRVAR(jointNameToIndex_doc,
 
 //getJointParentIndex
 PyDoc_STRVAR(getJointParentIndex_doc,
-	"Set the value of joint in world coordinate system\n"\
+	"Returns the index of the parent joint of the specified joint\n"\
 	"\n"\
-	"parentindex = figure.setJoint(jointName or index)\n"\
+	"parentindex = figure.getJointParentIndex(jointName or index)\n"\
 	"\n"\
 	"Parameters\n"\
 	"----------\n"\
 	"	jointName : string\n"\
-	"		The name of the joint you want to get\n"\
+	"		The name of the joint\n"\
 	"	index : int\n"\
-	"		The index of the joint you want to get\n"\
+	"		The index of the joint\n"\
 	"Returns\n"\
 	"-------\n"\
-	"	index : int\n"\
-	"		The index of joint\n");
-
-
+	"	parentindex : int\n"\
+	"		The parent joint index\n");
 
 
 //setMeshAlpha
@@ -314,6 +348,141 @@ PyDoc_STRVAR(getMeshAlpha_doc,
 	"	alpha : float\n"\
 	"		alpha values.\n");
 
+//getMeshPrimitiveType
+PyDoc_STRVAR(getMeshPrimitiveType_doc,
+	"Get the primitive type of Mesh.\n"\
+	"\n"\
+	"type = figure.getMeshPrimitiveType(meshName)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"	meshName : string (or mesh index : int)\n"\
+	"		string of mesh name or int of mesh index to get the value.\n	"\
+	"Returns\n"\
+	"-------\n"\
+	"	type : int\n"\
+	"		primirive type.\n");
+
+//setMeshPrimitiveType
+PyDoc_STRVAR(setMeshPrimitiveType_doc,
+	"Set the primitive type of Mesh.\n"\
+	"\n"\
+	"figure.setMeshPrimitiveType(meshName, mode)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"	meshName : string (or int)\n"\
+	"		string of mesh name or int of mesh index to set value.\n	"\
+	"	type : int\n"\
+	"		primitive type.\n"\
+	"			igeCore.GL_LINES\n"\
+	"			igeCore.GL_LINE_LOOP\n"\
+	"			igeCore.GL_LINE_STRIP\n"\
+	"			igeCore.GL_TRIANGLES\n"\
+	"			igeCore.GL_TRIANGLE_STRIP\n"\
+	"			igeCore.GL_TRIANGLE_FAN\n"\
+	"			igeCore.GL_QUADS\n"\
+	"			igeCore.GL_QUAD_STRIP\n"\
+	"\n");
+
+/*
+//getMeshWireframe
+PyDoc_STRVAR(getMeshWireframe_doc,
+	"Get whether the specified mesh is in wireframe rendering mode.\n"\
+	"\n"\
+	"mode = figure.getMeshWireframe(meshName)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"	meshName : string (or int)\n"\
+	"		string of mesh name or int of mesh index to get the value.\n	"\
+	"Returns\n"\
+	"-------\n"\
+	"	mode : bool\n"\
+	"		wireframe mode.\n");
+
+//setMeshWireframe
+PyDoc_STRVAR(setMeshWireframe_doc,
+	"Set whether the specified mesh is in wireframe rendering mode.\n"\
+	"\n"\
+	"This function can only set an incorrect wireframe.\n"\
+	"It's just for debug purpose.\n"\
+	"\n"\
+	"figure.setMeshWireframe(meshName, mode)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"	meshName : string (or int)\n"\
+	"		string of mesh name or int of mesh index to set value.\n	"\
+	"	mode : bool\n"\
+	"		wireframe mode.\n"\
+	"\n");
+*/
+
+
+PyDoc_STRVAR(getNumMeshTriangles_doc,
+	"Get the number of polygons in the mesh.\n"\
+	"\n"\
+	"number = figure.getNumMeshTriangles(meshName)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"	meshName : string (or int)\n"\
+	"		string of mesh name or int of mesh index to get the value.\n	"\
+	"Returns\n"\
+	"-------\n"\
+	"	number : int\n"\
+	"		number of triangles.\n");
+
+PyDoc_STRVAR(getNumMeshVertices_doc,
+	"Get the number of vertices in the mesh.\n"\
+	"\n"\
+	"number = figure.getNumMeshVertices(meshName)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"	meshName : string (or int)\n"\
+	"		string of mesh name or int of mesh index to get the value.\n	"\
+	"Returns\n"\
+	"-------\n"\
+	"	number : int\n"\
+	"		number of vertices.\n");
+
+
+
+//getShaderGenerator
+PyDoc_STRVAR(getShaderGenerator_doc,
+	"Get shaderGenerator.\n"\
+	"\n"\
+	"Returns the ShaderGenerator for the specified material.\n"\
+	"\n"\
+	"shaderGen = figure.getShaderGenerator(materialName)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"	materialName : string (or int)\n"\
+	"		string of material name or int of material index.\n	"\
+	"Returns\n"\
+	"-------\n"\
+	"	shaderGen : igeCore.shaderGenerator\n"\
+	"		shaderGenerator object.\n");
+
+//setShaderGenerator
+PyDoc_STRVAR(setShaderGenerator_doc,
+	"Set shaderGenerator.\n"\
+	"\n"\
+	"Set shaderGenerator to specified material and replace shader.\n"\
+	"\n"\
+	"figure.setShaderGenerator(materialName, shaderGen)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"	materialName : string (or int)\n"\
+	"		string of material name or int of material index.\n	"\
+	"	shaderGen : igeCore.shaderGenerator\n"\
+	"		shaderGenerator object.\n");
+
+
 //setMaterialParam
 PyDoc_STRVAR(setMaterialParam_doc,
 	"Set material parameters\n"\
@@ -437,8 +606,6 @@ PyDoc_STRVAR(setMaterialRenderState_doc,
 	"        Render state parameter");
 
 
-
-
 //setParentJoint
 PyDoc_STRVAR(setParentJoint_doc,
 	"Set the joint of another figure \n"\
@@ -458,17 +625,29 @@ PyDoc_STRVAR(setParentJoint_doc,
 PyDoc_STRVAR(getAABB_doc,
 	"Calclate axis aligned bounding box\n"\
 	"\n"\
-	"min, max, = figure.getMeshAABB(name or index)\n"\
+	"min, max, = figure.getAABB(name or index, space)\n"\
 	"\n"\
 	"Parameters\n"\
 	"----------\n"\
 	"    name : string  (optional)\n"\
-	"        The name of the mesh to calclate AABB\n"\
+	"        The name of the mesh to calclate AABB	\n"\
+	"        By omitting it, the entire figure will be calculated.\n"\
 	"    index : int  (optional)\n"\
 	"        Index number of mesh to calclate AABB\n"\
 	"        \n"\
 	"        If the parameter is not specified or -1 is specified, \n"\
 	"        the AABB of the entire figure is calculated. \n"\
+	"	space : int \n"\
+	"		-1 (default value)\n"\
+	"			Calculate AABB from vertex before animation\n"\
+	"		0 (igeCore.LocalSpace)\n"\
+	"			Calculate AABB after animation in locak space\n"\
+	"		1 (igeCore.WorldSpace)\n"\
+	"			Calculate AABB after animation in world space\n"\
+	"\n"\
+	"		This function is slow as it uses the CPU to compute all vertices.\n"\
+	"		Frequent use in real time is not recommended.\n"\
+	"\n"\
 	"Returns\n"\
 	"-------\n"\
 	"    min : vec3\n"\
@@ -508,6 +687,7 @@ PyDoc_STRVAR(getEmbeddedAnimationName_doc,
 	"    name : string\n"\
 	"        animnation file name	\n"\
 	"        None if the index is out of range");
+
 PyDoc_STRVAR(getJointName_doc,
 	"Get joint name.\n"\
 	"\n"\
@@ -524,6 +704,7 @@ PyDoc_STRVAR(getJointName_doc,
 	"    name : string\n"\
 	"        joint name	\n"\
 	"        None if the index is out of range");
+
 PyDoc_STRVAR(getMeshName_doc,
 	"Get mesh name.\n"\
 	"\n"\
@@ -558,3 +739,163 @@ PyDoc_STRVAR(getMaterialName_doc,
 	"        material name	\n"\
 	"        None if the index is out of range");
 
+
+PyDoc_STRVAR(getMeshIndex_doc,
+	"Get mesh indexNo from mesh name.\n"\
+	"\n"\
+	"index = figure.getMeshIndex(name)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"    name : string\n"\
+	"        mesh name\n"\
+	"Returns\n"\
+	"-------\n"\
+	"    index : int\n"\
+	"        mesh indexNo	\n"\
+	"        return -1 if mesh is not found");
+
+PyDoc_STRVAR(getMaterialIndex_doc,
+	"Get material indexNo from material name.\n"\
+	"\n"\
+	"index = figure.getMaterialIndex(name)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"    name : string\n"\
+	"        material name\n"\
+	"Returns\n"\
+	"-------\n"\
+	"    index : int\n"\
+	"        material indexNo	\n"\
+	"        return -1 if material is not found");
+
+PyDoc_STRVAR(getJointIndex_doc,
+	"Get joint indexNo from joint name.\n"\
+	"\n"\
+	"index = figure.getJointIndex(name)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"    name : string\n"\
+	"        joint name\n"\
+	"Returns\n"\
+	"-------\n"\
+	"    index : int\n"\
+	"        joint indexNo	\n"\
+	"        return -1 if joint is not found");
+
+//setVertexElements
+PyDoc_STRVAR(setVertexElements_doc,
+	"Set vertex data to the mesh object\n"\
+	"\n"\
+	"figure.setVertexElements(meshName or index, attributeType, data, offset)\n"\
+	"\n"\
+	"This method can't add new attributes or vertices like editableFigure.\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"    meshName : string\n"\
+	"        Registered mesh name to which you want to add vertex information\n"\
+	"    index : int\n"\
+	"        Registered mesh index to which you want to add vertex information\n"\
+	"    attributeType : int\n"\
+	"        Vertex element type\n"\
+	"        One of the following values\n"\
+	"\n"\
+	"        igeCore.ATTRIBUTE_ID_POSITION\n"\
+	"        igeCore.ATTRIBUTE_ID_NORMAL\n"\
+	"        igeCore.ATTRIBUTE_ID_TANGENT\n"\
+	"        igeCore.ATTRIBUTE_ID_BINORMAL\n"\
+	"        igeCore.ATTRIBUTE_ID_UV0\n"\
+	"        igeCore.ATTRIBUTE_ID_UV1\n"\
+	"        igeCore.ATTRIBUTE_ID_UV2\n"\
+	"        igeCore.ATTRIBUTE_ID_UV3\n"\
+	"        igeCore.ATTRIBUTE_ID_COLOR\n"\
+	"        igeCore.ATTRIBUTE_ID_BLENDINDICES\n"\
+	"        igeCore.ATTRIBUTE_ID_BLENDWEIGHT\n"\
+	"        igeCore.ATTRIBUTE_ID_PSIZE\n"\
+	"    data : tuple or list\n"\
+	"        Vertex elements\n"\
+	"    offset : int (optional)\n"\
+	"        Start position to add vertex elements\n"\
+	"Examples\n"\
+	"----------\n"\
+	"    efig.addMesh('meshA', 'materialA');\n"\
+	"    points = ((-1, 1, 0), (1, 1, 0), (-1, -1, 0), (1, -1, 0))\n"\
+	"    efig.setVertexElements('meshA', igeCore.ATTRIBUTE_ID_POSITION, points");
+
+//getVertexElements
+PyDoc_STRVAR(getVertexElements_doc,
+	"Get vertex data from the mesh object\n"\
+	"\n"\
+	"figure.getVertexElements(meshName or index, attributeType, offset, size)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"    meshName : string\n"\
+	"        Registered mesh name to which you want to add vertex information\n"\
+	"    index : int\n"\
+	"        Registered mesh index No to which you want to get vertex information\n"\
+	"    attributeType : int\n"\
+	"        Vertex element type\n"\
+	"        One of the following values\n"\
+	"\n"\
+	"        igeCore.ATTRIBUTE_ID_POSITION\n"\
+	"        igeCore.ATTRIBUTE_ID_NORMAL\n"\
+	"        igeCore.ATTRIBUTE_ID_TANGENT\n"\
+	"        igeCore.ATTRIBUTE_ID_BINORMAL\n"\
+	"        igeCore.ATTRIBUTE_ID_UV0\n"\
+	"        igeCore.ATTRIBUTE_ID_UV1\n"\
+	"        igeCore.ATTRIBUTE_ID_UV2\n"\
+	"        igeCore.ATTRIBUTE_ID_UV3\n"\
+	"        igeCore.ATTRIBUTE_ID_COLOR\n"\
+	"        igeCore.ATTRIBUTE_ID_BLENDINDICES\n"\
+	"        igeCore.ATTRIBUTE_ID_BLENDWEIGHT\n"\
+	"        igeCore.ATTRIBUTE_ID_PSIZE\n"\
+	"    offset : int (optional)\n"\
+	"        Start position to get vertex elements\n"\
+	"        0 if omitted.\n"\
+	"    size : int (optional)\n"\
+	"        Numver of vertex elements to get.\n"\
+	"        If omitted, all data after the offset.\n");
+
+//setTriangles
+PyDoc_STRVAR(setTriangles_doc,
+	"Set traiangle index data to the mesh object\n"\
+	"\n"\
+	"figure.setTriangles(meshName or index, data, offset)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"    meshName : string\n"\
+	"        Registered mesh name to which you want to add triangle index\n"\
+	"    index : string\n"\
+	"        Registered mesh index to which you want to add triangle index\n"\
+	"    data : tuple or list\n"\
+	"        Triangle index\n"\
+	"    offset : int (optional)\n"\
+	"        Start triangle position to add\n");
+
+//getTriangles
+PyDoc_STRVAR(getTriangles_doc,
+	"Get traiangle index data of the mesh object\n"\
+	"\n"\
+	"triangles = figure.getTriangles(meshName or index, data, offset, size)\n"\
+	"\n"\
+	"Parameters\n"\
+	"----------\n"\
+	"    meshName : string\n"\
+	"        Registered mesh name to which you want to get triangle index\n"\
+	"    index : string\n"\
+	"        Registered mesh index to which you want to get triangle index\n"\
+	"    offset : int (optional)\n"\
+	"        Start triangle position to get triangles\n"\
+	"    size : int (optional)\n"\
+	"        Number of triangles  to get\n"\
+	"Returns\n"\
+	"-------\n"\
+	"	triangles :tuple of integer\n"\
+	"		Triangle indices array\n");
+
+
